# update queries
SET = "set"  # sets an document's field to a new value
UNSET = "unset"  # unset a document's field
PUSH = "push"  # pushes an element into a document's array field
POP = "pop"  # removes either the first or last element in an array
INC = "inc"  # increments a document's numerical field
