# find queries
EQ = "eq"  # equals
NE = "ne"  # not equals
GT = "gt"  # greater than
LT = "lt"  # less than
GTE = "gte"  # greater than/equals
LTE = "lte"  # less than/equals
IN = "in"  # array contains
NIN = "nin"  # array not contains
